@@ -4,12 +4,10 @@ import matplotlib.pyplot as plt
 import pymc as pm
 import hddm
 import sys
-import table_print
+import kabuki
 
 from scipy.stats import scoreatpercentile
-from numpy import array, zeros, ones, empty
-from copy import deepcopy
-from time import time
+from scipy.stats.mstats import mquantiles
 
 def flip_errors(data):
     """Flip sign for lower boundary responses.
@@ -27,13 +25,28 @@ def flip_errors(data):
         return data
 
     # Copy data
-    data = np.array(data)
+    data = data.copy()
+
     # Flip sign for lower boundary response
     idx = data['response'] == 0
     data['rt'][idx] = -data['rt'][idx]
 
     return data
 
+def check_params_valid(**params):
+    a = params.get('a')
+    v = params.get('v')
+    z = params.get('z', .5)
+    t = params.get('t')
+    sv = params.get('sv', 0)
+    st = params.get('st', 0)
+    sz = params.get('sz', 0)
+
+    if (sv < 0) or (a <= 0) or (z < 0) or (z > 1) or (sz < 0) or (sz > 1) or (z+sz/2. > 1) or \
+    (z-sz/2. < 0) or (t-st/2. < 0) or (t < 0) or (st < 0):
+        return False
+    else:
+        return True
 
 def histogram(a, bins=10, range=None, normed=False, weights=None, density=None):
     """
@@ -271,7 +284,7 @@ def parse_config_file(fname, map=True, mcmc=False, data=None, samples=None, burn
         except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
             thin = 2
 
-    group_params = ['v', 'V', 'a', 'z', 'Z', 't', 'T']
+    group_params = ['v', 'sv', 'a', 'z', 'sz', 't', 'st']
 
     # Get depends
     depends = {}
@@ -291,18 +304,15 @@ def parse_config_file(fname, map=True, mcmc=False, data=None, samples=None, burn
 
     m.mcmc().sample(samples, burn=burn, thin=thin, verbose=verbose)
 
-    if only_group_stats:
-        print kabuki.analyze.print_group_stats(m.mc.stats())
-    else:
-        print kabuki.analyze.print_stats(m.mc.stats())
-
+    m.print_stats()
+    m.print_stats(fname='stats.txt')
     print "logp: %f" % m.mc.logp
     print "DIC: %f" % m.mc.dic
 
     if plot:
-        m.plot_posteriors()
+        m.plot_posteriors(save=True)
         print "Plotting posterior predictive..."
-        kabuki.analyze.plot_posterior_predictive(m, value_range=np.linspace(-3, 3, 100), savefig=True)
+        m.plot_posterior_predictive(save=True)
 
     return m
 
@@ -457,7 +467,7 @@ def hddm_parents_trace(model, obs_node, idx):
     function is used by ppd_test)
     """
     model.params_include.keys()
-    params = {'a': 0, 'v': 0, 't':0, 'z': 0.5, 'Z': 0, 'T': 0, 'V': 0}
+    params = {'a': 0, 'v': 0, 't':0, 'z': 0.5, 'sz': 0, 'st': 0, 'sv': 0}
     if not np.isscalar(idx):
         for (key, value) in params.iteritems():
             params[key] = np.ones(len(idx)) * value
@@ -493,6 +503,63 @@ def _gen_statistics():
 
     return statistics
 
+def gen_ppc_stats(quantiles = (10, 30, 50, 70, 90)):
+    """Generate default statistics for posterior predictive check on
+    RT data.
+
+    :Returns:
+        OrderedDict mapping statistic name -> function
+    """
+    from collections import OrderedDict
+
+    stats = OrderedDict()
+    stats['accuracy'] = lambda x: np.mean(x>0)
+
+    #upper bound stats
+    stats['mean_ub'] = lambda x: np.mean(x[x>0])
+    stats['std_ub'] = lambda x: np.std(x[x>0])
+    for q in quantiles:
+        key = str(q) + 'q'
+        stats[key+'_ub'] = lambda x, q=q: scoreatpercentile(x[x>0], q) if np.any(x>0) else np.nan
+
+    #lower bound stats
+    stats['mean_lb'] = lambda x: np.mean(x[x<0])
+    stats['std_lb'] = lambda x: np.std(x[x<0])
+    for q in quantiles:
+        key = str(q) + 'q'
+        stats[key+'_lb'] = lambda x, q=q: scoreatpercentile(np.abs(x[x<0]), q) if np.any(x<0) else np.nan
+
+    return stats
+
+
+def post_pred_check(model, **kwargs):
+    """Run posterior predictive check on a model.
+
+    :Arguments:
+        model : kabuki.Hierarchical
+            Kabuki model over which to compute the ppc on.
+
+    :Optional:
+        samples : int
+            How many samples to generate for each node.
+        bins : int
+            How many bins to use for computing the histogram.
+        evals : dict
+            User-defined evaluations of the statistics (by default 95 percentile and SEM).
+            :Example: {'percentile': scoreatpercentile}
+        plot : bool
+            Whether to plot the posterior predictive distributions.
+        progress_bar: bool
+            Display progress bar while sampling.
+
+
+    :Returns:
+        Hierarchical pandas.DataFrame with the different statistics.
+
+    """
+
+    stats = gen_ppc_stats()
+    return kabuki.analyze.post_pred_check(model, stats=stats, **kwargs)
 
 def plot_posteriors(model, **kwargs):
     """Generate posterior plots for each parameter.
@@ -509,320 +576,137 @@ def data_plot(data, nbins=50):
     plt.show()
 
 
-def quantiles_summary(hm, method, n_samples,
-                      quantiles = (10, 30, 50, 70, 90), sorted_idx = None,
-                      cdf_range  = (-5, 5)):
-    """
-    generate quantiles summary of data or samples
-    Input:
-        hm - hddm model
-        method - which parameters to use for the computation of quantiles
-            'observed' - quantiles from the observed data (not using any parameters)
-            'deviance' - quantiles of data sampled using the parameters that have maximum deviance
-            'samples'  - quantiles of data generated from samples from the posterior distrbution
-        qunaitles - the qunatiles to be generated
-        sorted_idx - the condtions will be sort according to sorted_idx.
-            if sorted_idx == None then they will be sorted according to the
-                accuracy achieved in each condition.
-        cdf_range (advanced) - the range of the cdf used to generate the estimated
-            quantiles.
+def _plot_posterior_quantiles_node(node, axis, quantiles=(.1, .3, .5, .7, .9),
+                                   samples=100, alpha=.75, hexbin=True,
+                                   value_range=(0, 5),
+                                   data_plot_kwargs=None, predictive_plot_kwargs=None):
+    """Plot posterior quantiles for a single node.
 
-    Ouput:
-        group_dict - a dictionary with group statistics
-        subj_dict - a list of dictionaries wih subjects statistcs
-        conds - the list of conditions sorted associated with the statistics
-        sorted_idx - the indices of the conditions such that:
-            if we set: keys = hm.params_dict['wfpt'].subj_nodes.keys()
-            than: keys[sorted_idx] == conds
+    :Arguments:
+
+        node : pymc.Node
+            Must be observable.
+
+        axis : matplotlib.axis handle
+            Axis to plot into.
+
+    :Optional:
+
+        value_range : numpy.ndarray
+            Range over which to evaluate the CDF.
+
+        samples : int (default=10)
+            Number of posterior samples to use.
+
+        alpha : float (default=.75)
+           Alpha (transparency) of posterior quantiles.
+
+        hexbin : bool (default=False)
+           Whether to plot posterior quantile density
+           using hexbin.
+
+        data_plot_kwargs : dict (default=None)
+           Forwarded to data plotting function call.
+
+        predictive_plot_kwargs : dict (default=None)
+           Forwareded to predictive plotting function call.
 
     """
 
-    #check if group model
-    if hm.is_group_model:
-        is_group = True
-        n_subj = hm._num_subjs
+    quantiles = np.asarray(quantiles)
+
+    axis.set_xlim(value_range)
+    axis.set_ylim((0, 1))
+
+    theo = np.empty((2, 2, len(quantiles), samples))
+    for sample in range(samples):
+        kabuki.analyze._parents_to_random_posterior_sample(node)
+        theo[:, :, :, sample] = node.quantiles(quantiles)
+
+    if hexbin:
+        if predictive_plot_kwargs is None:
+            predictive_plot_kwargs = {'gridsize': 75, 'bins': 'log', 'extent': (value_range[0], value_range[1], 0, 1)}
+        axis.hexbin(theo[:,0,:,:].flatten(), theo[:,1,:,:].flatten(), label='post pred lb', **predictive_plot_kwargs)
     else:
-        n_subj = 1
-        is_group = False
-
-    #check method
-    if method == 'observed':
-        n_samples = 1
-        len_trace = 1
-    elif method == 'samples':
-        db =hm.mc.db
-        name = list(hm.mc.stochastics)[0].__name__
-        len_trace = db.trace(name).length()
-        params = hddm.generate.gen_rand_params()
-    elif method == 'deviance':
-        db =hm.mc.db
-        params = hddm.generate.gen_rand_params()
-    else:
-        raise ValueError, "unkown method"
-
-    #Init
-    n_q = len(quantiles)
-    wfpt_dict = hm.params_dict['wfpt'].subj_nodes
-    conds = wfpt_dict.keys()
-    n_conds = len(conds)
-    temp_dict = {}
-    group_dict = {}; subj_dict = [None]*n_subj;
-    count = zeros((n_conds * 2, n_samples))
-    temp_dict['q'] = zeros((n_conds * 2, n_samples, n_q))
-    temp_dict['prob'] = zeros((n_conds * 2, n_samples))
-    temp_dict['n'] = zeros((n_conds*2, n_samples))
-    group_dict = deepcopy(temp_dict)
-    for i_subj in range(n_subj):
-        subj_dict[i_subj] = deepcopy(temp_dict)
-
-    #loop over subjs
-    for i_subj in range(n_subj):
-        #get wfpt nodes
-        if is_group:
-            wfpt = [x[i_subj] for x in hm.params_dict['wfpt'].subj_nodes.values()]
-        else:
-            wfpt = hm.params_dict['wfpt'].subj_nodes.values()
-        #loop over conds
-        for i_cond in range(n_conds):
-            wfpt_params = dict(wfpt[i_cond].parents)
-            data_len = len(wfpt[i_cond].value)
-            for i_sample in xrange(n_samples):
-                #get rt
-                if method == 'observed':
-                    rt = wfpt[i_cond].value
-                else:
-                    #get sample_idx
-                    if method == 'deviance':
-                        sample_idx  = np.argmin(db.trace('deviance')[:])
-                    else:
-                        sample_idx = ((len_trace - 1) // (n_samples - 1)) * i_sample
-                    #get params
-                    for key in params.iterkeys():
-                        if isinstance(wfpt_params[key], pm.Node):
-                            node = wfpt_params[key]
-                            name = node.__name__
-                            params[key] = db.trace(name)[sample_idx]
-                    #generate rt
-                    rt = hddm.generate.gen_rts(params, samples=data_len, range_=cdf_range,
-                                               dt=1e-3, intra_sv=1., structured=False,
-                                               subj_idx=None, method='cdf')
-                #loop over responses
-                for i_resp in range(2):
-                    cond_ind = i_cond * 2 + i_resp
-                    if i_resp==0:
-                        t_rt = rt[rt > 0]
-                    else:
-                        t_rt = rt[rt < 0]
-                    #get n_q
-                    if len(t_rt)>=n_q:
-                        t_quan = [scoreatpercentile(abs(t_rt),x) for x in quantiles]
-                        subj_dict[i_subj]['q'][cond_ind, i_sample, :] = t_quan
-                        group_dict['q'][cond_ind, i_sample, :] += t_quan
-                        count[cond_ind, i_sample] += 1
-                    else:
-                        subj_dict[i_subj]['q'][cond_ind, i_sample, :] = np.NaN
-                    t_prob = 1.*len(t_rt) / len(rt)
-                    subj_dict[i_subj]['prob'][cond_ind, i_sample] = t_prob
-                    group_dict['prob'][cond_ind, i_sample] += t_prob
-
-                    subj_dict[i_subj]['n'][cond_ind, i_sample] = len(t_rt)
-                    group_dict['n'][cond_ind, i_sample] = len(t_rt)
+        if predictive_plot_kwargs is None:
+            predictive_plot_kwargs = {'alpha': .75}
+        axis.plot(theo[0,0,:,:], theo[0,1,:,:], label='post pred lb', color='b', **predictive_plot_kwargs)
+        axis.plot(theo[1,0,:,:], theo[1,1,:,:], label='post pred ub', color='r', **predictive_plot_kwargs)
 
 
+    # Plot data
+    data = node.value
+    color = 'w' if hexbin else 'k'
+    if data_plot_kwargs is None:
+        data_plot_kwargs = {'color': color, 'lw': 2., 'marker': 'o', 'markersize': 7}
 
-    #compute group values
-    group_dict['prob'] /= n_subj
-    for i_c in range(n_conds * 2):
-        for i_sample in xrange(n_samples):
-            if count[i_c, i_sample] > 0:
-                group_dict['q'][i_c, i_sample,:] /= count[i_c, i_sample]
-            else:
-                group_dict['q'][i_c, i_sample,:] = np.NaN
-    #sort
-    if sorted_idx == None:
-        sorted_idx = np.argsort(group_dict['prob'][:,0]).flatten()
-    group_dict['prob'] = group_dict['prob'][sorted_idx,:]
-    group_dict['q'] = group_dict['q'][sorted_idx,:,:]
-    group_dict['n'] = group_dict['n'][sorted_idx,:]
-    for i_subj in range(n_subj):
-        subj_dict[i_subj]['prob'] = subj_dict[i_subj]['prob'][sorted_idx,:]
-        subj_dict[i_subj]['q'] = subj_dict[i_subj]['q'][sorted_idx,:, :]
-        subj_dict[i_subj]['n'] = subj_dict[i_subj]['n'][sorted_idx, :]
+    if len(data) != 0:
+        p_upper = np.mean(data>0)
+        q_lower = mquantiles(-data[data<0], quantiles)
+        q_upper = mquantiles(data[data>0], quantiles)
 
-    conds = array(conds)[(sorted_idx // 2)]
-    return group_dict, subj_dict, conds, sorted_idx
+        axis.plot(q_lower, quantiles*(1-p_upper), **data_plot_kwargs)
+        axis.plot(q_upper, quantiles*p_upper, **data_plot_kwargs)
 
+    axis.set_xlabel('RT')
+    axis.set_ylabel('Prob respond')
+    axis.set_ylim(bottom=0) # Likelihood and histogram can only be positive
 
+def plot_posterior_quantiles(model, **kwargs):
+    """Plot posterior predictive quantiles.
+
+    :Arguments:
+
+        model : HDDM model
+
+    :Optional:
+
+        value_range : numpy.ndarray
+            Range over which to evaluate the CDF.
+
+        samples : int (default=10)
+            Number of posterior samples to use.
+
+        alpha : float (default=.75)
+           Alpha (transparency) of posterior quantiles.
+
+        hexbin : bool (default=False)
+           Whether to plot posterior quantile density
+           using hexbin.
+
+        data_plot_kwargs : dict (default=None)
+           Forwarded to data plotting function call.
+
+        predictive_plot_kwargs : dict (default=None)
+           Forwareded to predictive plotting function call.
+
+        columns : int (default=3)
+            How many columns to use for plotting the subjects.
+
+        save : bool (default=False)
+            Whether to save the figure to a file.
+
+        path : str (default=None)
+            Save figure into directory prefix
 
-def _draw_qp_plot(dict, conds, conds_to_plot, title_str, samples_summary,
-                  marker, handle = None):
-    """
-    draw QP plot
-    Input:
-        dict - subject/group dictionary from qunatiles_summary
-        conds - conditions from quantiles_summary
-        conds_to_plot - a list of conds to plot
-        title_str - the title of the plot
-        samples_summary - sould we plot only samples sammury and not the samples themselves
-        maarker - marker used in plot
-        handle - axes's handle
     """
 
-    colors  = ['b', 'g', 'c', 'm', 'r', 'y', 'k']
-    n_q = dict['q'].shape[2]
-    n_conds = len(conds)//2
-    n_samples = dict['prob'].shape[1]
-    #group plot
-    if handle == None:
-        f = plt.figure()
-        handle = f.add_subplot(111)
-    color_counter = -1
-    for i_c in range(n_conds):
-        #check if we need to plot this condition
-        if i_c not in conds_to_plot:
-            continue
-        color_counter += 1
-        #loop over correct and incorrect reponses
-        for i_resp in range(2):
-            if i_resp == 0:
-                idx = i_c
-            else:
-                idx = (n_conds * 2) - 1 - i_c
-            #get color of marker
-            cc = colors[color_counter % len(colors)]
-            #plot it
-            if samples_summary:
-                ok_idx = ~np.isnan(dict['prob'][idx,:])
-                if sum(ok_idx) == 0:
-                    continue
-                mean_prob = np.mean(dict['prob'][idx,ok_idx])
-                xerr = np.empty((2,n_q))
-                xerr[0,:] = mean_prob - scoreatpercentile(dict['prob'][idx,ok_idx], 2.5)
-                xerr[1,:] = scoreatpercentile(dict['prob'][idx,ok_idx], 97.5) - mean_prob
+    if 'value_range' not in kwargs:
+        rt = np.abs(model.data['rt'])
+        kwargs['value_range'] = (np.min(rt.min()-.2, 0), rt.max()-.2)
 
-                mean_q = np.mean(dict['q'][idx,ok_idx,:],0)
-                yerr = np.empty((2,n_q))
-                yerr[0,:] = array([scoreatpercentile(x, 2.5) for x in dict['q'][idx,ok_idx,:].T]) - mean_q
-                yerr[1,:] = mean_q - array([scoreatpercentile(x, 97.5) for x in dict['q'][idx,ok_idx,:].T])
-                handle.errorbar(ones(n_q)*mean_prob, mean_q, yerr=yerr, xerr=xerr,
-                                marker=marker, fmt=cc)
-
-            else:
-                for i_sample in range(n_samples):
-                    if np.isnan(dict['prob'][idx, i_sample]):
-                        continue
-                    handle.plot(ones(n_q)*dict['prob'][idx,i_sample],
-                                      dict['q'][idx,i_sample,:],'%s%s' % (cc, marker))
-
-    #add title and labels
-    if title_str != None:
-        plt.gcf().canvas.set_window_title(title_str)
-        plt.title(title_str)
-        plt.xlabel('probability')
-        plt.ylabel('RT')
-        plt.xlim([-0.05, 1.05])
-
-    handle.get_figure().canvas.draw()
-    return handle
+    kabuki.analyze.plot_posterior_predictive(model,
+                                             plot_func=_plot_posterior_quantiles_node,
+                                             required_method='quantiles',
+                                             **kwargs)
 
 
-def qp_plot(hm, quantiles = (10, 30, 50, 70, 90), plot_subj=True,
-            split_func=lambda x:0, method = None, samples_summary=True,
-            n_samples=50, cdf_range=(-5, 5)):
-    """
-    generate a quantile-probability plot
-    Input:
-        hm - hddm model
-        qunatiles - the quantiles that are going to be displyed
-        plot_subj - generate a QP plot for each subject as well
-        split_func - split the plot into several plots according to the output of
-            the function
-        method - which values will be used to compute the estimated quantiles.
-            may be one of the followings:
-            -'none'   : display only observed quantiles
-            -'deviance': use the point of minimum deviance
-            -'samples': samples n_samples and plot them all (not recommended)
-            -'samples_summary': samples n_samples and plot the mean and 95 credible set of them
-        n_samples - see method
-        cdf_range (advanced) - the range of the cdf used to generate the estimated
-            quantiles.
-
-        TODO:
-            there should be an option to use the average value of the parameters to create samples.
-            it make much more sense than 'deviance'
-    """
-
-    #### compute empirical quantiles
-    #Init
-    n_q = len(quantiles)
-    if hm.is_group_model:
-        is_group = True
-        n_subj = hm._num_subjs
-    else:
-        n_subj = 1
-        is_group = False
-
-    #get group and subj dicts
-    obs_group_d, obs_subj_d, conds, sorted_idx = \
-    quantiles_summary(hm, 'observed', n_samples = 1, quantiles = quantiles)
-
-    #get splits
-    keys = [split_func(x) for x in conds]
-    dic  = {}
-    for i_s in range(len(keys)):
-        if dic.has_key(keys[i_s]):
-            dic[keys[i_s]].append(i_s)
-        else:
-            dic[keys[i_s]] = [i_s]
-    splits = dic.values()
-    splits_keys = dic.keys()
-
-    #plot
-    g_handles = [None]*len(splits)
-    s_handles = [[None]*n_subj for x in range(len(splits))]
-    for i_s in range(len(splits)):
-        g_handles[i_s] = _draw_qp_plot(obs_group_d, conds, splits[i_s],
-                                title_str = "QP group (%s)" % splits_keys[i_s],
-                                samples_summary=False, marker = 'd')
-        if not plot_subj:
-            continue
-        for i_subj in range(n_subj):
-            s_handles[i_s][i_subj] = _draw_qp_plot(obs_subj_d[i_subj], conds, splits[i_s],
-                                           title_str = "QP %d (%s)" % (i_subj, splits_keys[i_s]),
-                                           samples_summary=False, marker = 'd')
-
-    if (method == None) or (method == 'none'):
-        return
-    elif (method == 'deviance'):
-        quantiles_method = 'deviance'
-    elif method == 'samples':
-        quantiles_method = 'samples'
-    else:
-        raise ValueError, "unknown method"
-
-    #### compute estimated quantiles
-
-    #get group and subj dicts
-    print "getting quantiles summary of samples"
-    i_t = time();
-    sim_group_d, sim_subj_d, conds, sorted_idx = \
-    quantiles_summary(hm, method=method, n_samples=n_samples,
-                      quantiles=quantiles, sorted_idx=sorted_idx)
-    print "took %d seconds to prepare quantiles" % (time() - i_t)
-    sys.stdout.flush()
-
-    #plot
-    for i_s in range(len(splits)):
-        _draw_qp_plot(sim_group_d, conds, splits[i_s],
-                      title_str=None, samples_summary=samples_summary,
-                      marker='o', handle=g_handles[i_s])
-        if not plot_subj:
-            continue
-        for i_subj in range(n_subj):
-            _draw_qp_plot(sim_subj_d[i_subj], conds, splits[i_s],
-                          title_str = None, samples_summary=samples_summary,
-                          marker = 'o', handle = s_handles[i_s][i_subj])
+def create_test_model(samples=5000, burn=1000, subjs=1, size=100):
+    data, params = hddm.generate.gen_rand_data(subjs=subjs, size=size)
+    m = hddm.HDDM(data)
+    m.sample(samples, burn=burn)
 
+    return m
 
 if __name__ == "__main__":
     import doctest
